#include<iostream>
#include<fstream>
#include<sstream>
#include<string.h>
#include<threadhelper.h>
#include<TSCtimestamp.h>
#include<clocktime.h>

struct WorkerData
{
    unsigned long tnr;
    clockid_t clock;
};

const char *sym[32];

void *workerproc(void *ptr)
{
    WorkerData *data = reinterpret_cast<WorkerData *>(ptr);
    std::clog << "Thread " << data->tnr << " tid " << gettid() << std::endl;
    int cpuid = (int)(data->tnr);
    runon(cpuid);
    std::ostringstream fnamestream;
    fnamestream << "drift" << sym[data->clock] << '_' << data->tnr << ".dat";
    std::string fname = fnamestream.str();
    std::clog << data->tnr << " writes " << fname << std::endl;
    std::ofstream driftfile;
    driftfile.open(fname.c_str());

    struct timespec checkinterval;
    checkinterval.tv_sec = 0;
    checkinterval.tv_nsec = 25000000;

    time64_t firstmono = getclocktime(data->clock);
    TSCTimestamp_t firsttsc = getTSCtimestamp();
    time64_t lastmono = firstmono;
    TSCTimestamp_t lasttsc = firsttsc;

    for(int i=0;i<500;i++)
    {
	nanosleep(&checkinterval,0);
	time64_t currentmono = getclocktime(data->clock);
	TSCTimestamp_t currenttsc = getTSCtimestamp();
	time64_t monorel = currentmono-firstmono;
        TSCTimestamp_t tscrel = currenttsc-firsttsc;
	uint64_t diffmono=currentmono-lastmono;
	uint64_t difftsc =currenttsc -lasttsc;
        driftfile  << i << '\t'
		   << monorel << '\t' << tscrel
		   << '\t' << diffmono << '\t' << difftsc
		   << '\t' << (double)(difftsc)/double(diffmono)
		   << std::endl;
	
	lastmono=currentmono;
	lasttsc =currenttsc;
    }
    driftfile.close();
}

int main(int argc,char **argv)
{
    const unsigned int NWORKERS=2;
    pthread_t workers[NWORKERS];
    WorkerData data[NWORKERS];
    int ret[NWORKERS];
    for(int i=0;i<32;i++) { sym[i]=""; }
    sym[CLOCK_MONOTONIC]="MONO";
    sym[CLOCK_REALTIME]="REAL";
    clockid_t clk_id=CLOCK_MONOTONIC;
    if(argc>1)
    {
	char **arg = argv+1;
	if(strcmp(*arg,"-M")==0)
	{
	    clk_id=CLOCK_MONOTONIC;
	    std::clog << "CLOCK_MONOTONIC" << std::endl;
	}
	else if(strcmp(*arg,"-R")==0)
	{
	    clk_id=CLOCK_REALTIME;
	    std::clog << "CLOCK_REALTIME" << std::endl;
	}
    }

    for(unsigned int t=0;t<NWORKERS;t++)
    {
	data[t].tnr=t;
	data[t].clock = clk_id;
	ret[t] = pthread_create(workers+t,NULL,workerproc,data+t);
	std::clog << "pthread_create " << t << " returns " 
		  << ret[t] << std::endl;
    }
    for(unsigned int t=0;t<NWORKERS;t++)
    {
	pthread_join(workers[t],NULL);
	std::clog << t << " is joined" << std::endl;
    }
}

